import logging
import threading
from typing import List, Dict, Tuple, Union, Any

from sfdevtools.data_cache.DPage import DPage
from sfdevtools.data_cache.DStrategy import DStrategy
from sfdevtools.data_cache.DComponents import StrategyInsight, TS_Order, TS_Trade
import sfdevtools.data_cache.DTimelyCache as TimelyCache
import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc

class DCache(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__pages: Dict[str, DPage] = None # key: QC Symbol ID, value: data page
        self.__strategy: DStrategy = DStrategy()
        self.__page_mutex: threading.Lock = threading.Lock()
        self.__update_cb: Any = None
        self.__cache_name: str = None

    def init_component(self
                       , logger: logging.Logger
                       , update_cb
                       , cache_name: str
                       , max_hist_orders: int = 1000
                       , max_hist_trades: int = 1000
                       , max_hist_si: int = 1000) -> None:
        self.__logger = logger
        self.__cache_name = cache_name
        self.__update_cb = update_cb
        self.__strategy.init_component(logger=self.__logger
                                       , update_cb=self.__update_cb
                                       , strategy_name=cache_name
                                       , get_parent_dcache_fun=self.__get_self
                                       , max_hist_orders=max_hist_orders
                                       , max_hist_si=max_hist_si
                                       , max_hist_trades=max_hist_trades)
        self.__pages = dict()

    def save_si(self, si: List[StrategyInsight]) -> bool:
        return self.__strategy.save_si(si=si)

    def is_latest_si(self, si: List[StrategyInsight]) -> bool:
        return self.__strategy.is_latest_si(si=si)

    def is_si_exist(self, si: List[StrategyInsight]) -> bool:
        return self.__strategy.is_si_exist(si=si)

    def is_ci_exist(self, ci: Dict[str, Any]) -> bool:
        return self.__strategy.is_ci_exist(ci=ci)

    def get_ci(self) -> Dict[str, Any]:
        return self.__strategy.get_ci()

    def get_ci_id(self) -> str:
        return self.__strategy.get_ci_id()

    def is_order_exist(self, order: TS_Order) -> bool:
        return self.__strategy.is_order_exist(order=order)

    def is_db_record_id_exist(self, order: TS_Order) -> bool:
        return self.__strategy.is_db_record_id_exist(order=order)

    def get_order_latest_snapshot(self, platform_order_id: str) -> Union[bool, TS_Order]:
        return self.__strategy.get_order_latest_snapshot(platform_order_id=platform_order_id)

    def get_db_record_id(self, order_id: str) -> Union[bool, str]:
        return self.__strategy.get_db_record_id(order_id=order_id)

    def get_order(self, order_id: str) -> Union[bool, TS_Order]:
        return self.__strategy.get_order(order_id=order_id)

    def save_ci(self, ci: Dict[str, Any]) -> None:
        self.__strategy.save_ci(ci=ci)

    def save_ci_id(self, ci_id: str) -> None:
        self.__strategy.save_ci_id(ci_id=ci_id)

    def save_order(self, order: TS_Order) -> None:
        self.__strategy.save_order(order=order)

    def save_orders(self, orders: List[TS_Order]) -> None:
        self.__strategy.save_orders(orders=orders)

    def save_db_record_id(self, order: TS_Order, db_record_id: str) -> None:
        self.__strategy.save_db_record_id(order=order, db_record_id=db_record_id)

    def is_trade_exist(self, trade: TS_Trade) -> bool:
        return self.__strategy.is_trade_exist(trade=trade)

    def get_trade(self, trade_id: str) -> Union[bool, TS_Trade]:
        return self.__strategy.get_trade(trade_id=trade_id)

    def save_trade(self, trade: TS_Trade) -> None:
        self.__strategy.save_trade(trade=trade)

    def save_trades(self, trades: List[TS_Trade]) -> None:
        self.__strategy.save_trades(trades=trades)

    def save_fid(self
                 , msg_id: str
                 , msg_type: ts_cop_pb2.Cop.MsgType
                 , fid_num: int
                 , fid_value: Any) -> None:

        # handle strategy level
        if msg_type == ts_cop_pb2.Cop.MsgType.Strategy:
            self.__strategy.save_fid(fid_num=fid_num, fid_value=fid_value)
            return

        # handle page level
        dpage = self.get_create_dpage(page_id=msg_id)
        dpage.save_fid(fid_num=fid_num, fid_value=fid_value)

    def get_cache_name(self) -> str:
        return self.__cache_name

    def get_page_fids(self
                 , page_id: str) -> List[Union[int, Any]]:
        dpage = self.get_create_dpage(page_id=msg_id)
        return dpage.get_fids()

    def get_strategy_fids(self) -> List[Union[int, Any]]:
        return self.__strategy.get_fids()

    def get_fid(self
                , msg_id: str
                , msg_type: ts_cop_pb2.Cop.MsgType
                , fid_num: int) -> Union[bool, Union[bool, Any]]:
        # handle strategy level
        if msg_type == ts_cop_pb2.Cop.MsgType.Strategy:
            return self.__strategy.get_fid(fid_num=fid_num)

        # handle page level
        dpage = self.get_create_dpage(page_id=msg_id)
        return dpage.get_fid(fid_num=fid_num)

    def get_page_name(self) -> List[str]:
        with self.__page_mutex:
            return [page_id for page_id, page in self.__pages.items()]

    def get_page_count(self) -> int:
        with self.__page_mutex:
            return len(self.__pages)

    def get_strgy(self) -> DStrategy:
        return self.__strategy

    def get_create_dpage(self, page_id: str) -> DPage:
        with self.__page_mutex:
            if page_id in self.__pages:
                return self.__pages[page_id]
            else:
                new_dpage = DPage()
                new_dpage.init_component(logger=self.__logger
                                         , update_cb=self.__update_cb
                                         , page_id=page_id
                                         , get_parent_dcache_fun=self.__get_self
                                         , save_other_map=dict()
                                         , get_other_map=dict())
                self.__pages[page_id] = new_dpage
                return new_dpage

    def __get_self(self) -> Any:
        return self



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: peacock.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='peacock.proto',
  package='peacock',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\rpeacock.proto\x12\x07peacock\".\n\x17GetDataByStrategyId_Msg\x12\x13\n\x0bstrategy_id\x18\x01 \x01(\t\"\x80\x01\n\x19GetDataByStrategyId_Reply\x12\x0b\n\x03\x63\x66g\x18\x01 \x03(\t\x12\n\n\x02\x63i\x18\x02 \x03(\t\x12\n\n\x02si\x18\x03 \x01(\t\x12\x10\n\x08ord_snap\x18\x04 \x01(\t\x12\x10\n\x08ord_hist\x18\x05 \x01(\t\x12\x0b\n\x03trd\x18\x06 \x01(\t\x12\r\n\x05price\x18\x07 \x01(\t\"c\n\x19GetBacktestWarmupData_Msg\x12\x13\n\x0bstrategy_id\x18\x01 \x01(\t\x12\x0e\n\x06msg_id\x18\x02 \x01(\t\x12\x10\n\x08msg_type\x18\x03 \x01(\x05\x12\x0f\n\x07\x66id_num\x18\x04 \x01(\x05\"+\n\x1bGetBacktestWarmupData_Reply\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\"*\n\x10GetUSLEIData_Msg\x12\x16\n\x0elookback_month\x18\x01 \x01(\x05\"\x91\x01\n\x12GetUSLEIData_Reply\x12\x46\n\x0fus_lei_data_map\x18\x01 \x03(\x0b\x32-.peacock.GetUSLEIData_Reply.UsLeiDataMapEntry\x1a\x33\n\x11UsLeiDataMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\\\n\x18RemoveStrategyConfig_Msg\x12\x15\n\rstrategy_name\x18\x01 \x01(\t\x12\x0f\n\x07is_live\x18\x02 \x01(\x08\x12\x18\n\x10strategy_id_list\x18\x03 \x03(\t\"#\n\x13RemoveOldConfig_Msg\x12\x0c\n\x04\x64\x61ys\x18\x01 \x01(\x05\"\x1c\n\x0b\x44ummy_Reply\x12\r\n\x05\x64ummy\x18\x01 \x01(\t\"?\n\x15GetStrategyConfig_Msg\x12\x15\n\rstrategy_name\x18\x01 \x01(\t\x12\x0f\n\x07is_live\x18\x02 \x01(\x08\";\n\x17GetStrategyConfig_Reply\x12\x0e\n\x06status\x18\x01 \x01(\t\x12\x10\n\x08\x63\x66g_list\x18\x02 \x03(\t\"1\n\x16SaveStrategyConfig_Msg\x12\x17\n\x0fstrategy_config\x18\x01 \x01(\t\"*\n\x18SaveStrategyConfig_Reply\x12\x0e\n\x06status\x18\x01 \x01(\t\">\n\x13SaveFileToColud_Msg\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x14\n\x0c\x66ile_content\x18\x02 \x01(\t\"Z\n\x1aSaveRefDataPriceFromQC_Msg\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x14\n\x0c\x66ile_content\x18\x02 \x01(\t\x12\x13\n\x0b\x61\x63tion_mode\x18\x03 \x01(\t\"U\n\x15SaveRefDataFromQC_Msg\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x14\n\x0c\x66ile_content\x18\x02 \x01(\t\x12\x13\n\x0b\x61\x63tion_mode\x18\x03 \x01(\t\"\x1a\n\tNoneReply\x12\r\n\x05reply\x18\x01 \x01(\t\"\x17\n\x04Ping\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x17\n\x04Pong\x12\x0f\n\x07message\x18\x01 \x01(\t2\xe8\x06\n\x07Peacock\x12\x43\n\x0fSaveFileToColud\x12\x1c.peacock.SaveFileToColud_Msg\x1a\x12.peacock.NoneReply\x12Q\n\x16SaveRefDataPriceFromQC\x12#.peacock.SaveRefDataPriceFromQC_Msg\x1a\x12.peacock.NoneReply\x12G\n\x11SaveRefDataFromQC\x12\x1e.peacock.SaveRefDataFromQC_Msg\x1a\x12.peacock.NoneReply\x12X\n\x12SaveStrategyConfig\x12\x1f.peacock.SaveStrategyConfig_Msg\x1a!.peacock.SaveStrategyConfig_Reply\x12U\n\x11GetStrategyConfig\x12\x1e.peacock.GetStrategyConfig_Msg\x1a .peacock.GetStrategyConfig_Reply\x12O\n\x14RemoveStrategyConfig\x12!.peacock.RemoveStrategyConfig_Msg\x1a\x14.peacock.Dummy_Reply\x12\x45\n\x0fRemoveOldConfig\x12\x1c.peacock.RemoveOldConfig_Msg\x1a\x14.peacock.Dummy_Reply\x12[\n\x13GetDataByStrategyId\x12 .peacock.GetDataByStrategyId_Msg\x1a\".peacock.GetDataByStrategyId_Reply\x12\x61\n\x15GetBacktestWarmupData\x12\".peacock.GetBacktestWarmupData_Msg\x1a$.peacock.GetBacktestWarmupData_Reply\x12\x46\n\x0cGetUSLEIData\x12\x19.peacock.GetUSLEIData_Msg\x1a\x1b.peacock.GetUSLEIData_Reply\x12+\n\x0bHealthCheck\x12\r.peacock.Ping\x1a\r.peacock.Pongb\x06proto3'
)




_GETDATABYSTRATEGYID_MSG = _descriptor.Descriptor(
  name='GetDataByStrategyId_Msg',
  full_name='peacock.GetDataByStrategyId_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='peacock.GetDataByStrategyId_Msg.strategy_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=72,
)


_GETDATABYSTRATEGYID_REPLY = _descriptor.Descriptor(
  name='GetDataByStrategyId_Reply',
  full_name='peacock.GetDataByStrategyId_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cfg', full_name='peacock.GetDataByStrategyId_Reply.cfg', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ci', full_name='peacock.GetDataByStrategyId_Reply.ci', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='si', full_name='peacock.GetDataByStrategyId_Reply.si', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ord_snap', full_name='peacock.GetDataByStrategyId_Reply.ord_snap', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ord_hist', full_name='peacock.GetDataByStrategyId_Reply.ord_hist', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trd', full_name='peacock.GetDataByStrategyId_Reply.trd', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='price', full_name='peacock.GetDataByStrategyId_Reply.price', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=75,
  serialized_end=203,
)


_GETBACKTESTWARMUPDATA_MSG = _descriptor.Descriptor(
  name='GetBacktestWarmupData_Msg',
  full_name='peacock.GetBacktestWarmupData_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_id', full_name='peacock.GetBacktestWarmupData_Msg.strategy_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg_id', full_name='peacock.GetBacktestWarmupData_Msg.msg_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg_type', full_name='peacock.GetBacktestWarmupData_Msg.msg_type', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fid_num', full_name='peacock.GetBacktestWarmupData_Msg.fid_num', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=205,
  serialized_end=304,
)


_GETBACKTESTWARMUPDATA_REPLY = _descriptor.Descriptor(
  name='GetBacktestWarmupData_Reply',
  full_name='peacock.GetBacktestWarmupData_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='peacock.GetBacktestWarmupData_Reply.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=306,
  serialized_end=349,
)


_GETUSLEIDATA_MSG = _descriptor.Descriptor(
  name='GetUSLEIData_Msg',
  full_name='peacock.GetUSLEIData_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lookback_month', full_name='peacock.GetUSLEIData_Msg.lookback_month', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=351,
  serialized_end=393,
)


_GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY = _descriptor.Descriptor(
  name='UsLeiDataMapEntry',
  full_name='peacock.GetUSLEIData_Reply.UsLeiDataMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='peacock.GetUSLEIData_Reply.UsLeiDataMapEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='peacock.GetUSLEIData_Reply.UsLeiDataMapEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=490,
  serialized_end=541,
)

_GETUSLEIDATA_REPLY = _descriptor.Descriptor(
  name='GetUSLEIData_Reply',
  full_name='peacock.GetUSLEIData_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='us_lei_data_map', full_name='peacock.GetUSLEIData_Reply.us_lei_data_map', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=396,
  serialized_end=541,
)


_REMOVESTRATEGYCONFIG_MSG = _descriptor.Descriptor(
  name='RemoveStrategyConfig_Msg',
  full_name='peacock.RemoveStrategyConfig_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='peacock.RemoveStrategyConfig_Msg.strategy_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_live', full_name='peacock.RemoveStrategyConfig_Msg.is_live', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_id_list', full_name='peacock.RemoveStrategyConfig_Msg.strategy_id_list', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=543,
  serialized_end=635,
)


_REMOVEOLDCONFIG_MSG = _descriptor.Descriptor(
  name='RemoveOldConfig_Msg',
  full_name='peacock.RemoveOldConfig_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='days', full_name='peacock.RemoveOldConfig_Msg.days', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=637,
  serialized_end=672,
)


_DUMMY_REPLY = _descriptor.Descriptor(
  name='Dummy_Reply',
  full_name='peacock.Dummy_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dummy', full_name='peacock.Dummy_Reply.dummy', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=674,
  serialized_end=702,
)


_GETSTRATEGYCONFIG_MSG = _descriptor.Descriptor(
  name='GetStrategyConfig_Msg',
  full_name='peacock.GetStrategyConfig_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='peacock.GetStrategyConfig_Msg.strategy_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_live', full_name='peacock.GetStrategyConfig_Msg.is_live', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=704,
  serialized_end=767,
)


_GETSTRATEGYCONFIG_REPLY = _descriptor.Descriptor(
  name='GetStrategyConfig_Reply',
  full_name='peacock.GetStrategyConfig_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='peacock.GetStrategyConfig_Reply.status', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cfg_list', full_name='peacock.GetStrategyConfig_Reply.cfg_list', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=769,
  serialized_end=828,
)


_SAVESTRATEGYCONFIG_MSG = _descriptor.Descriptor(
  name='SaveStrategyConfig_Msg',
  full_name='peacock.SaveStrategyConfig_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_config', full_name='peacock.SaveStrategyConfig_Msg.strategy_config', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=830,
  serialized_end=879,
)


_SAVESTRATEGYCONFIG_REPLY = _descriptor.Descriptor(
  name='SaveStrategyConfig_Reply',
  full_name='peacock.SaveStrategyConfig_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='peacock.SaveStrategyConfig_Reply.status', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=881,
  serialized_end=923,
)


_SAVEFILETOCOLUD_MSG = _descriptor.Descriptor(
  name='SaveFileToColud_Msg',
  full_name='peacock.SaveFileToColud_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_name', full_name='peacock.SaveFileToColud_Msg.file_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file_content', full_name='peacock.SaveFileToColud_Msg.file_content', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=925,
  serialized_end=987,
)


_SAVEREFDATAPRICEFROMQC_MSG = _descriptor.Descriptor(
  name='SaveRefDataPriceFromQC_Msg',
  full_name='peacock.SaveRefDataPriceFromQC_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_name', full_name='peacock.SaveRefDataPriceFromQC_Msg.file_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file_content', full_name='peacock.SaveRefDataPriceFromQC_Msg.file_content', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='action_mode', full_name='peacock.SaveRefDataPriceFromQC_Msg.action_mode', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=989,
  serialized_end=1079,
)


_SAVEREFDATAFROMQC_MSG = _descriptor.Descriptor(
  name='SaveRefDataFromQC_Msg',
  full_name='peacock.SaveRefDataFromQC_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_name', full_name='peacock.SaveRefDataFromQC_Msg.file_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file_content', full_name='peacock.SaveRefDataFromQC_Msg.file_content', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='action_mode', full_name='peacock.SaveRefDataFromQC_Msg.action_mode', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1081,
  serialized_end=1166,
)


_NONEREPLY = _descriptor.Descriptor(
  name='NoneReply',
  full_name='peacock.NoneReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='reply', full_name='peacock.NoneReply.reply', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1168,
  serialized_end=1194,
)


_PING = _descriptor.Descriptor(
  name='Ping',
  full_name='peacock.Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='peacock.Ping.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1196,
  serialized_end=1219,
)


_PONG = _descriptor.Descriptor(
  name='Pong',
  full_name='peacock.Pong',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='peacock.Pong.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1221,
  serialized_end=1244,
)

_GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY.containing_type = _GETUSLEIDATA_REPLY
_GETUSLEIDATA_REPLY.fields_by_name['us_lei_data_map'].message_type = _GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY
DESCRIPTOR.message_types_by_name['GetDataByStrategyId_Msg'] = _GETDATABYSTRATEGYID_MSG
DESCRIPTOR.message_types_by_name['GetDataByStrategyId_Reply'] = _GETDATABYSTRATEGYID_REPLY
DESCRIPTOR.message_types_by_name['GetBacktestWarmupData_Msg'] = _GETBACKTESTWARMUPDATA_MSG
DESCRIPTOR.message_types_by_name['GetBacktestWarmupData_Reply'] = _GETBACKTESTWARMUPDATA_REPLY
DESCRIPTOR.message_types_by_name['GetUSLEIData_Msg'] = _GETUSLEIDATA_MSG
DESCRIPTOR.message_types_by_name['GetUSLEIData_Reply'] = _GETUSLEIDATA_REPLY
DESCRIPTOR.message_types_by_name['RemoveStrategyConfig_Msg'] = _REMOVESTRATEGYCONFIG_MSG
DESCRIPTOR.message_types_by_name['RemoveOldConfig_Msg'] = _REMOVEOLDCONFIG_MSG
DESCRIPTOR.message_types_by_name['Dummy_Reply'] = _DUMMY_REPLY
DESCRIPTOR.message_types_by_name['GetStrategyConfig_Msg'] = _GETSTRATEGYCONFIG_MSG
DESCRIPTOR.message_types_by_name['GetStrategyConfig_Reply'] = _GETSTRATEGYCONFIG_REPLY
DESCRIPTOR.message_types_by_name['SaveStrategyConfig_Msg'] = _SAVESTRATEGYCONFIG_MSG
DESCRIPTOR.message_types_by_name['SaveStrategyConfig_Reply'] = _SAVESTRATEGYCONFIG_REPLY
DESCRIPTOR.message_types_by_name['SaveFileToColud_Msg'] = _SAVEFILETOCOLUD_MSG
DESCRIPTOR.message_types_by_name['SaveRefDataPriceFromQC_Msg'] = _SAVEREFDATAPRICEFROMQC_MSG
DESCRIPTOR.message_types_by_name['SaveRefDataFromQC_Msg'] = _SAVEREFDATAFROMQC_MSG
DESCRIPTOR.message_types_by_name['NoneReply'] = _NONEREPLY
DESCRIPTOR.message_types_by_name['Ping'] = _PING
DESCRIPTOR.message_types_by_name['Pong'] = _PONG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetDataByStrategyId_Msg = _reflection.GeneratedProtocolMessageType('GetDataByStrategyId_Msg', (_message.Message,), {
  'DESCRIPTOR' : _GETDATABYSTRATEGYID_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.GetDataByStrategyId_Msg)
  })
_sym_db.RegisterMessage(GetDataByStrategyId_Msg)

GetDataByStrategyId_Reply = _reflection.GeneratedProtocolMessageType('GetDataByStrategyId_Reply', (_message.Message,), {
  'DESCRIPTOR' : _GETDATABYSTRATEGYID_REPLY,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.GetDataByStrategyId_Reply)
  })
_sym_db.RegisterMessage(GetDataByStrategyId_Reply)

GetBacktestWarmupData_Msg = _reflection.GeneratedProtocolMessageType('GetBacktestWarmupData_Msg', (_message.Message,), {
  'DESCRIPTOR' : _GETBACKTESTWARMUPDATA_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.GetBacktestWarmupData_Msg)
  })
_sym_db.RegisterMessage(GetBacktestWarmupData_Msg)

GetBacktestWarmupData_Reply = _reflection.GeneratedProtocolMessageType('GetBacktestWarmupData_Reply', (_message.Message,), {
  'DESCRIPTOR' : _GETBACKTESTWARMUPDATA_REPLY,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.GetBacktestWarmupData_Reply)
  })
_sym_db.RegisterMessage(GetBacktestWarmupData_Reply)

GetUSLEIData_Msg = _reflection.GeneratedProtocolMessageType('GetUSLEIData_Msg', (_message.Message,), {
  'DESCRIPTOR' : _GETUSLEIDATA_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.GetUSLEIData_Msg)
  })
_sym_db.RegisterMessage(GetUSLEIData_Msg)

GetUSLEIData_Reply = _reflection.GeneratedProtocolMessageType('GetUSLEIData_Reply', (_message.Message,), {

  'UsLeiDataMapEntry' : _reflection.GeneratedProtocolMessageType('UsLeiDataMapEntry', (_message.Message,), {
    'DESCRIPTOR' : _GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY,
    '__module__' : 'peacock_pb2'
    # @@protoc_insertion_point(class_scope:peacock.GetUSLEIData_Reply.UsLeiDataMapEntry)
    })
  ,
  'DESCRIPTOR' : _GETUSLEIDATA_REPLY,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.GetUSLEIData_Reply)
  })
_sym_db.RegisterMessage(GetUSLEIData_Reply)
_sym_db.RegisterMessage(GetUSLEIData_Reply.UsLeiDataMapEntry)

RemoveStrategyConfig_Msg = _reflection.GeneratedProtocolMessageType('RemoveStrategyConfig_Msg', (_message.Message,), {
  'DESCRIPTOR' : _REMOVESTRATEGYCONFIG_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.RemoveStrategyConfig_Msg)
  })
_sym_db.RegisterMessage(RemoveStrategyConfig_Msg)

RemoveOldConfig_Msg = _reflection.GeneratedProtocolMessageType('RemoveOldConfig_Msg', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEOLDCONFIG_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.RemoveOldConfig_Msg)
  })
_sym_db.RegisterMessage(RemoveOldConfig_Msg)

Dummy_Reply = _reflection.GeneratedProtocolMessageType('Dummy_Reply', (_message.Message,), {
  'DESCRIPTOR' : _DUMMY_REPLY,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.Dummy_Reply)
  })
_sym_db.RegisterMessage(Dummy_Reply)

GetStrategyConfig_Msg = _reflection.GeneratedProtocolMessageType('GetStrategyConfig_Msg', (_message.Message,), {
  'DESCRIPTOR' : _GETSTRATEGYCONFIG_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.GetStrategyConfig_Msg)
  })
_sym_db.RegisterMessage(GetStrategyConfig_Msg)

GetStrategyConfig_Reply = _reflection.GeneratedProtocolMessageType('GetStrategyConfig_Reply', (_message.Message,), {
  'DESCRIPTOR' : _GETSTRATEGYCONFIG_REPLY,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.GetStrategyConfig_Reply)
  })
_sym_db.RegisterMessage(GetStrategyConfig_Reply)

SaveStrategyConfig_Msg = _reflection.GeneratedProtocolMessageType('SaveStrategyConfig_Msg', (_message.Message,), {
  'DESCRIPTOR' : _SAVESTRATEGYCONFIG_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.SaveStrategyConfig_Msg)
  })
_sym_db.RegisterMessage(SaveStrategyConfig_Msg)

SaveStrategyConfig_Reply = _reflection.GeneratedProtocolMessageType('SaveStrategyConfig_Reply', (_message.Message,), {
  'DESCRIPTOR' : _SAVESTRATEGYCONFIG_REPLY,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.SaveStrategyConfig_Reply)
  })
_sym_db.RegisterMessage(SaveStrategyConfig_Reply)

SaveFileToColud_Msg = _reflection.GeneratedProtocolMessageType('SaveFileToColud_Msg', (_message.Message,), {
  'DESCRIPTOR' : _SAVEFILETOCOLUD_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.SaveFileToColud_Msg)
  })
_sym_db.RegisterMessage(SaveFileToColud_Msg)

SaveRefDataPriceFromQC_Msg = _reflection.GeneratedProtocolMessageType('SaveRefDataPriceFromQC_Msg', (_message.Message,), {
  'DESCRIPTOR' : _SAVEREFDATAPRICEFROMQC_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.SaveRefDataPriceFromQC_Msg)
  })
_sym_db.RegisterMessage(SaveRefDataPriceFromQC_Msg)

SaveRefDataFromQC_Msg = _reflection.GeneratedProtocolMessageType('SaveRefDataFromQC_Msg', (_message.Message,), {
  'DESCRIPTOR' : _SAVEREFDATAFROMQC_MSG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.SaveRefDataFromQC_Msg)
  })
_sym_db.RegisterMessage(SaveRefDataFromQC_Msg)

NoneReply = _reflection.GeneratedProtocolMessageType('NoneReply', (_message.Message,), {
  'DESCRIPTOR' : _NONEREPLY,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.NoneReply)
  })
_sym_db.RegisterMessage(NoneReply)

Ping = _reflection.GeneratedProtocolMessageType('Ping', (_message.Message,), {
  'DESCRIPTOR' : _PING,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.Ping)
  })
_sym_db.RegisterMessage(Ping)

Pong = _reflection.GeneratedProtocolMessageType('Pong', (_message.Message,), {
  'DESCRIPTOR' : _PONG,
  '__module__' : 'peacock_pb2'
  # @@protoc_insertion_point(class_scope:peacock.Pong)
  })
_sym_db.RegisterMessage(Pong)


_GETUSLEIDATA_REPLY_USLEIDATAMAPENTRY._options = None

_PEACOCK = _descriptor.ServiceDescriptor(
  name='Peacock',
  full_name='peacock.Peacock',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1247,
  serialized_end=2119,
  methods=[
  _descriptor.MethodDescriptor(
    name='SaveFileToColud',
    full_name='peacock.Peacock.SaveFileToColud',
    index=0,
    containing_service=None,
    input_type=_SAVEFILETOCOLUD_MSG,
    output_type=_NONEREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SaveRefDataPriceFromQC',
    full_name='peacock.Peacock.SaveRefDataPriceFromQC',
    index=1,
    containing_service=None,
    input_type=_SAVEREFDATAPRICEFROMQC_MSG,
    output_type=_NONEREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SaveRefDataFromQC',
    full_name='peacock.Peacock.SaveRefDataFromQC',
    index=2,
    containing_service=None,
    input_type=_SAVEREFDATAFROMQC_MSG,
    output_type=_NONEREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SaveStrategyConfig',
    full_name='peacock.Peacock.SaveStrategyConfig',
    index=3,
    containing_service=None,
    input_type=_SAVESTRATEGYCONFIG_MSG,
    output_type=_SAVESTRATEGYCONFIG_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetStrategyConfig',
    full_name='peacock.Peacock.GetStrategyConfig',
    index=4,
    containing_service=None,
    input_type=_GETSTRATEGYCONFIG_MSG,
    output_type=_GETSTRATEGYCONFIG_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveStrategyConfig',
    full_name='peacock.Peacock.RemoveStrategyConfig',
    index=5,
    containing_service=None,
    input_type=_REMOVESTRATEGYCONFIG_MSG,
    output_type=_DUMMY_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveOldConfig',
    full_name='peacock.Peacock.RemoveOldConfig',
    index=6,
    containing_service=None,
    input_type=_REMOVEOLDCONFIG_MSG,
    output_type=_DUMMY_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDataByStrategyId',
    full_name='peacock.Peacock.GetDataByStrategyId',
    index=7,
    containing_service=None,
    input_type=_GETDATABYSTRATEGYID_MSG,
    output_type=_GETDATABYSTRATEGYID_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetBacktestWarmupData',
    full_name='peacock.Peacock.GetBacktestWarmupData',
    index=8,
    containing_service=None,
    input_type=_GETBACKTESTWARMUPDATA_MSG,
    output_type=_GETBACKTESTWARMUPDATA_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUSLEIData',
    full_name='peacock.Peacock.GetUSLEIData',
    index=9,
    containing_service=None,
    input_type=_GETUSLEIDATA_MSG,
    output_type=_GETUSLEIDATA_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='HealthCheck',
    full_name='peacock.Peacock.HealthCheck',
    index=10,
    containing_service=None,
    input_type=_PING,
    output_type=_PONG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_PEACOCK)

DESCRIPTOR.services_by_name['Peacock'] = _PEACOCK

# @@protoc_insertion_point(module_scope)



import logging
import threading
import signal

import sfdevtools.observability.log_helper as lh
import sfdevtools.devTools.HDTimer as HDTimer
import sfdevtools.devTools.MsgQ as MsgQ
import sfdevtools.devTools.AtomicData as AtomicData
import sfdevtools.devTools.DiscontinueSeelp as DSleep

class ThreadQ(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self._msg_q: MsgQ.MsgQ = None
        self._d_sleep: DSleep.DiscontinueSeelp = None
        self._is_running: AtomicData.AtomicData = None
        self._th: threading = None
        self._min_insertable_q_size: int = None

    def init_component(self
                       , logger: logging.Logger
                       , sleep_time_ms: int = 5000
                       , min_insertable_q_size: int = 100):
        self.__logger = logger
        self._msg_q = MsgQ.MsgQ()
        self._d_sleep = DSleep.DiscontinueSeelp(sleep_time_ms=sleep_time_ms)
        self._min_insertable_q_size = min_insertable_q_size
        self._is_running = AtomicData.AtomicData()
        self._is_running.save(user_data=True)

        self._th = threading.Thread(target=self._main)
        signal.signal(signal.SIGINT, self._cleanup)

    def start_th(self) -> None:
        self._th.start()

    def stop_th(self) -> None:
        if not self._is_running.get():
            return

        self._is_running.save(user_data=False)
        self._th.join()

    def _cleanup(self, signum, frame):
        self.stop_th()
        exit(0)

    # override
    def _is_time_to_aweak(self) -> bool:
        return self._is_running.get()\
            or not self._is_time_to_sleep()

    # override
    def _is_time_to_sleep(self) -> bool:
        return (self._msg_q.size()) < self._min_insertable_q_size

    # override
    def _user_main(self):
        raise NotImplementedError

    def _main(self):
        self.__logger.info("Start")

        while self._is_running.get():
            if not self._is_running.get():
                break

            if self._is_time_to_sleep():
                self._d_sleep.sleep(user_fun=self._is_time_to_aweak)
            # TODO - add user processing logic
            self._user_main()

            if not self._is_running.get():
                break

        self.__logger.info("End")



import logging
import datetime
from typing import Dict, List, Any, Union
import pandas as pd

def get_datetime_safe(data: Any) -> datetime.datetime:
    if isinstance(data, str):
        return datetime.datetime.strptime(data, "%y-%m-%d")
    elif isinstance(data, datetime.datetime):
        return data
    elif isinstance(data, datetime.date):
        return datetime.datetime(data.year, data.month, data.day)

class IDMapp_Ele(object):
    def __init__(self):
        self.listing_id: str = ""
        self.code: str = ""
        self.start_date: datetime.datetime = None
        self.end_date: datetime.datetime = None

    def __str__(self):
        return f"{self.code},{self.listing_id},{self.start_date},{self.end_date}"

    @staticmethod
    def get_row_key() -> str:
        return "code"

    @staticmethod
    def create_ele(row: Dict):
        ele = IDMapp_Ele()

        ele.code = row["code"]
        ele.listing_id = row["listing_id"]
        ele.start_date = get_datetime_safe(data=row["start_date"])
        ele.end_date = get_datetime_safe(data=row["end_date"])

        return ele

class CompanyMap_Ele(object):
    def __init__(self):
        self.listing_id: str = ""
        self.company_id: str = ""
        self.company_name: str = ""
        self.start_date: datetime.datetime = None
        self.end_date: datetime.datetime = None

    def __str__(self):
        return f"{self.listing_id},{self.company_id},{self.company_name}"

    @staticmethod
    def get_row_key() -> str:
        return "listing_id"

    @staticmethod
    def create_ele(row: Dict):
        ele = CompanyMap_Ele()

        ele.listing_id = row["listing_id"]
        ele.company_id = row["company_id"]
        ele.company_name = row["company_name"]
        ele.start_date = row["start_date"]
        ele.end_date = row["end_date"]

        return ele

class GroupMap_Ele(object):
    def __init__(self):
        self.listing_id: str = ""
        self.ref_industry_code: str = ""
        self.ref_industry_group: str = ""
        self.ref_sector_code: str = ""
        self.start_date: datetime.datetime = None
        self.end_date: datetime.datetime = None

    def __str__(self):
        return f"{self.listing_id},{self.ref_industry_code},{self.ref_industry_group},{self.ref_sector_code},{self.start_date},{self.end_date}"

    @staticmethod
    def get_row_key() -> str:
        return "listing_id"

    @staticmethod
    def create_ele(row: Dict):
        ele = GroupMap_Ele()

        ele.listing_id = row["listing_id"]
        ele.ref_industry_code = row["ref_industry_code"]
        ele.ref_industry_group = row["ref_industry_group"]
        ele.ref_sector_code = row["ref_sector_code"]
        ele.start_date = get_datetime_safe(data=row["start_date"])
        ele.end_date = get_datetime_safe(data=row["end_date"])

        return ele




import datetime
import json

import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc
from sfdevtools.data_cache.DComponents import StrategyInsight, TS_Order, TS_Trade
from sfdevtools.data_cache.DPage import DPage
from sfdevtools.data_cache.DStrategy import DStrategy

def to_timestamp_safe(value) -> float:
    if isinstance(value, datetime.datetime):
        return value.timestamp()
    else:
        return value

def conv_strg_2_cop(strgy: DStrategy) -> ts_cop_pb2.Cop:
    cop: ts_cop_pb2.Cop = ts_cop_pb2.Cop()

    data_list: List[Union[int, Any]] = strgy.get_fids()
    # handle normal fids
    for ele in data_list:
        fid_num = ele[0]
        value = ele[1]

        if fid_num <= 20000:
            if fid_num <= 10000:
                cop.int32_map[fid_num] = value
            else:
                cop.int64_map[fid_num] = value
        elif fid_num <=40000:
            if fid_num <= 30000:
                cop.float_map[fid_num] = value
            else:
                cop.double_map[fid_num] = value
        elif fid_num <= 50000:
            cop.string_map[fid_num] = value
        elif fid_num <= 70000:
            cop.bool_map[fid_num] = value
        elif fid_num <= 90000:
            if fid_num <= 80000:
                cop.data_map[fid_num].d_int32_data.value = value[0]
                cop.data_map[fid_num].d_int32_data.time = value[1]
            else:
                cop.data_map[fid_num].d_int64_data.value = value[0]
                cop.data_map[fid_num].d_int64_data.time = value[1]
        elif fid_num <= 110000:
            if fid_num <= 100000:
                cop.data_map[fid_num].d_float_data.value = value[0]
                cop.data_map[fid_num].d_float_data.time = value[1]
            else:
                cop.data_map[fid_num].d_double_data.value = value[0]
                cop.data_map[fid_num].d_double_data.time = value[1]
        elif fid_num <= 120000:
            cop.data_map[fid_num].d_string_data.value = value[0]
            cop.data_map[fid_num].d_string_data.time = value[1]
        elif fid_num <= 130000:
            cop.data_map[fid_num].d_bool_data.value = value[0]
            cop.data_map[fid_num].d_bool_data.time = value[1]

    # ci
    for ci in strgy.get_all_ci():
        ci["created"] = to_timestamp_safe(ci["created"])
        ci["last_update"] = to_timestamp_safe(ci["last_update"])
        cop.ci_map[ts_cop_pb2.FidNum.CI].ci_list.append(ts_cop_pb2.CI(value=json.dumps(ci)))
    # si
    for si in strgy.get_all_si():
        si_ele = dconv.conv_SI_2_cop_si(si=si)
        cop.si_map[ts_cop_pb2.Cop.FidNum.SI].si_list.append(si_ele)
    # order
    for order in strgy.get_orders(last_n=10):
        ord_ele = dconv.conv_TS_Order_2_cop_order(order=order)
        cop.order_map[ts_cop_pb2.Cop.FidNum.Order].order_list.append(ord_ele)
    # trade
    for trd in strgy.get_trades(last_n=10):
        trd_ele = dconv.conv_TS_Trade_2_cop_trade(trd=trd)
        cop.trade_map[ts_cop_pb2.Cop.FidNum.Trade].trade_list.append(trd_ele)

    return cop

def conv_page_2_cop(dpage: DPage) -> ts_cop_pb2.Cop:
    cop: ts_cop_pb2.Cop = ts_cop_pb2.Cop()

    data_list: List[Union[int, Any]] = dpage.get_fids()
    for ele in data_list:
        fid_num = ele[0]
        value = ele[1]

        if fid_num <= 20000:
            if fid_num <= 10000:
                cop.int32_map[fid_num] = value
            else:
                cop.int64_map[fid_num] = value
        elif fid_num <=40000:
            if fid_num <= 30000:
                cop.float_map[fid_num] = value
            else:
                cop.double_map[fid_num] = value
        elif fid_num <= 50000:
            cop.string_map[fid_num] = value
        elif fid_num <= 70000:
            cop.bool_map[fid_num] = value
        elif fid_num <= 90000:
            if fid_num <= 80000:
                cop.data_map[fid_num].d_int32_data.value = value[0]
                cop.data_map[fid_num].d_int32_data.time = value[1]
            else:
                cop.data_map[fid_num].d_int64_data.value = value[0]
                cop.data_map[fid_num].d_int64_data.time = value[1]
        elif fid_num <= 110000:
            if fid_num <= 100000:
                cop.data_map[fid_num].d_float_data.value = value[0]
                cop.data_map[fid_num].d_float_data.time = value[1]
            else:
                cop.data_map[fid_num].d_double_data.value = value[0]
                cop.data_map[fid_num].d_double_data.time = value[1]
        elif fid_num <= 120000:
            cop.data_map[fid_num].d_string_data.value = value[0]
            cop.data_map[fid_num].d_string_data.time = value[1]
        elif fid_num <= 130000:
            cop.data_map[fid_num].d_bool_data.value = value[0]
            cop.data_map[fid_num].d_bool_data.time = value[1]

    return cop

def conv_TS_Order_2_cop_order(order: TS_Order) -> ts_cop_pb2.TSOrder:
    ord_ele = ts_cop_pb2.TSOrder()

    ord_ele.symbol = order.symbol
    ord_ele.symbol_id = order.symbol_id
    ord_ele.quantity = order.quantity
    ord_ele.fill_quantity = order.fill_quantity
    ord_ele.parent_id = order.parent_id
    ord_ele.order_id = order.order_id
    ord_ele.platform_order_id = order.platform_order_id
    ord_ele.created = to_timestamp_safe(order.created)
    ord_ele.last_update = to_timestamp_safe(order.last_update)
    ord_ele.strategy_name = order.strategy_name
    ord_ele.parent_strategy_name = order.parent_strategy_name
    ord_ele.live_mode = order.live_mode
    ord_ele.strategy_id = order.strategy_id
    ord_ele.order_status = order.order_status
    ord_ele.execution_broker = order.execution_broker
    ord_ele.clearing_broker = order.clearing_broker
    ord_ele.account = order.account
    ord_ele.price = order.price
    ord_ele.fee = order.fee
    ord_ele.fee_ccy = order.fee_ccy
    ord_ele.exchange = order.exchange
    ord_ele.ccy = order.ccy

    return ord_ele

def conv_TS_Trade_2_cop_trade(trd: TS_Trade) -> ts_cop_pb2.TSTrade:
    trd_ele = ts_cop_pb2.TSTrade()

    trd_ele.symbol = trd.symbol
    trd_ele.symbol_id = trd.symbol_id
    trd_ele.quantity = trd.quantity
    trd_ele.parent_id = trd.parent_id
    trd_ele.trade_id = trd.trade_id
    trd_ele.created = to_timestamp_safe(trd.created)
    trd_ele.last_update = to_timestamp_safe(trd.last_update)
    trd_ele.strategy_name = trd.strategy_name
    trd_ele.parent_strategy_name = trd.parent_strategy_name
    trd_ele.live_mode = trd.live_mode
    trd_ele.strategy_id = trd.strategy_id
    trd_ele.trade_status = trd.trade_status
    trd_ele.execution_broker = trd.execution_broker
    trd_ele.clearing_broker = trd.clearing_broker
    # trd_ele.settlement_date = trd.settlement_date # we don't have information from QC for now
    trd_ele.fx_ratio = trd.fx_ratio
    trd_ele.account = trd.account
    trd_ele.price = trd.price
    trd_ele.fee = trd.fee
    trd_ele.fee_ccy = trd.fee_ccy
    trd_ele.exchange = trd.exchange
    trd_ele.ccy = trd.ccy

    return trd_ele

def conv_SI_2_cop_si(si: StrategyInsight) -> ts_cop_pb2.SI:
    si_ele = ts_cop_pb2.SI()

    si_ele.symbol = si.symbol
    si_ele.symbol_id = si.symbol_id
    si_ele.ratio = si.ratio
    si_ele.parent_id = si.parent_id
    si_ele.si_id = si.si_id
    si_ele.created = to_timestamp_safe(si.created)
    si_ele.last_update = to_timestamp_safe(si.last_update)
    si_ele.strategy_name = si.strategy_name
    si_ele.parent_strategy_name = si.parent_strategy_name
    si_ele.live_mode = si.live_mode
    si_ele.strategy_id = si.strategy_id

    return si_ele

def conv_cop_order_2_TS_Order(ord_ele: ts_cop_pb2.TSOrder) -> TS_Order:
    order = TS_Order()

    order.symbol = ord_ele.symbol
    order.symbol_id = ord_ele.symbol_id
    order.quantity = ord_ele.quantity
    order.fill_quantity = ord_ele.fill_quantity
    order.parent_id = ord_ele.parent_id
    order.order_id = ord_ele.order_id
    order.platform_order_id = ord_ele.platform_order_id
    order.created = datetime.datetime.fromtimestamp(ord_ele.created)
    order.last_update = datetime.datetime.fromtimestamp(ord_ele.last_update)
    order.strategy_name = ord_ele.strategy_name
    order.parent_strategy_name = ord_ele.parent_strategy_name
    order.live_mode = ord_ele.live_mode
    order.strategy_id = ord_ele.strategy_id
    order.order_status = ord_ele.order_status
    order.execution_broker = ord_ele.execution_broker
    order.clearing_broker = ord_ele.clearing_broker
    order.account = ord_ele.account
    order.price = ord_ele.price
    order.fee = ord_ele.fee
    order.fee_ccy = ord_ele.fee_ccy
    order.exchange = ord_ele.exchange
    order.ccy = ord_ele.ccy

    return order

def conv_cop_trade_2_TS_Trade(trd_ele: ts_cop_pb2.TSTrade) -> TS_Trade:
    trd = TS_Trade()

    trd.symbol = trd_ele.symbol
    trd.symbol_id = trd_ele.symbol_id
    trd.quantity = trd_ele.quantity
    trd.parent_id = trd_ele.parent_id
    trd.trade_id = trd_ele.trade_id
    trd.created = datetime.datetime.fromtimestamp(trd_ele.created)
    trd.last_update = datetime.datetime.fromtimestamp(trd_ele.last_update)
    trd.strategy_name = trd_ele.strategy_name
    trd.parent_strategy_name = trd_ele.parent_strategy_name
    trd.live_mode = trd_ele.live_mode
    trd.strategy_id = trd_ele.strategy_id
    trd.trade_status = trd_ele.trade_status
    trd.execution_broker = trd_ele.execution_broker
    trd.clearing_broker = trd_ele.clearing_broker
    trd.fx_ratio = trd_ele.fx_ratio
    trd.account = trd_ele.account
    trd.price = trd_ele.price
    trd.fee = trd_ele.fee
    trd.fee_ccy = trd_ele.fee_ccy
    trd.exchange = trd_ele.exchange
    trd.ccy = trd_ele.ccy

    return trd

def conv_cop_si_2_SI(si_ele: ts_cop_pb2.SI) -> StrategyInsight:
    si = StrategyInsight(symbol=si_ele.symbol, ratio=si_ele.ratio)

    si.symbol = si_ele.symbol
    si.symbol_id = si_ele.symbol_id
    si.ratio = si_ele.ratio
    si.parent_id = si_ele.parent_id
    si.si_id = si_ele.si_id
    si.created = datetime.datetime.fromtimestamp(si_ele.created)
    si.last_update = datetime.datetime.fromtimestamp(si_ele.last_update)
    si.strategy_name = si_ele.strategy_name
    si.parent_strategy_name = si_ele.parent_strategy_name
    si.live_mode = si_ele.live_mode
    si.strategy_id = si_ele.strategy_id

    return si



#region imports
from AlgorithmImports import *
#endregion

import logging
from typing import List, Dict, Any
import json
import datetime

import sfdevtools.data_control.TSDataCtrl as TSDataCtrl
import sfdevtools.data_control.TSBackgroundCtrl as TSBackgroundCtrl
from sfdevtools.data_cache.DCacheManager import DCacheManager
import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc
from sfdevtools.data_cache.DComponents import StrategyInsight, TS_Order, TS_Trade
import sfdevtools.data_cache.DConverter as dconv

class TSDataManager(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__ts_datactrl_pub: TSDataCtrl = None
        self.__ts_datactrl_sub: TSDataCtrl = None
        self.__cop_cb = None
        self.__pub_topic: str = ""
        self.__output_dcache_m: DCacheManager = None
        self.__bg_ctrl: TSBackgroundCtrl.TSBackgroundCtrl = None
        self.__is_active_bg: bool = False

        self.__data_convert_map = {
            ts_cop_pb2.Cop.FidNum.SI: self.__convert_si
            , ts_cop_pb2.Cop.FidNum.Order: self.__convert_order
            , ts_cop_pb2.Cop.FidNum.Trade: self.__convert_trade
        }

    def init_component(self
                       , logger: logging.Logger
                       , ts_datactrl_pub: TSDataCtrl.TSDataCtrl
                       , sender_name: str
                       , pub_topic: str
                       , ts_datactrl_sub: TSDataCtrl.TSDataCtrl
                       , output_dcache_m: DCacheManager
                       , bg_ctrl: TSBackgroundCtrl.TSBackgroundCtrl
                       , is_active_bg: bool) -> None:
        self.__logger = logger
        # prepare publisher
        self.__ts_datactrl_pub = ts_datactrl_pub
        self.__sender_name = sender_name
        self.__pub_topic = pub_topic
        # prepare subscriber
        self.__ts_datactrl_sub = ts_datactrl_sub
        # prepare cache
        self.__output_dcache_m = output_dcache_m
        # prepare backgroup control
        self.__bg_ctrl = bg_ctrl
        self.__is_active_bg = is_active_bg

    def send_ci(self, msg_id: str, ci_list: List[Dict], is_save_to_cache: bool = True) -> None:
        cop = ts_cop_pb2.Cop()
        for ci in ci_list:
            cop.data_map[ts_cop_pb2.Cop.FidNum.CI].ci_data.ci_list.append(ts_cop_pb2.CI(value=json.dumps(ci)))

            if is_save_to_cache and self.__is_active_bg:
                # save to output cache
                self.__output_dcache_m.save_ci(cache_name=self.__sender_name, ci=ci)

        self.__ts_datactrl_pub.send_cop(msg_id=msg_id
                                        , msg_type=ts_cop_pb2.Cop.MsgType.Strategy
                                        , topic=self.__pub_topic
                                        , cop=cop)

    def send_si(self, msg_id:str, si_list: List[StrategyInsight], is_save_to_cache: bool = True) -> None:
        cop = ts_cop_pb2.Cop()
        for si in si_list:
            si_ele = dconv.conv_SI_2_cop_si(si=si)

            cop.data_map[ts_cop_pb2.Cop.FidNum.SI].si_data.si_list.append(si_ele)

        if is_save_to_cache and self.__is_active_bg:
            # save to output cache
            self.__output_dcache_m.save_si(cache_name=self.__sender_name, si=si_list)

        self.__ts_datactrl_pub.send_cop(msg_id=msg_id
                                        , msg_type=ts_cop_pb2.Cop.MsgType.Strategy
                                        , topic=self.__pub_topic
                                        , cop=cop)

    def send_order(self, msg_id: str, order_list: List[TS_Order], is_save_to_cache: bool = True) -> None:
        cop = ts_cop_pb2.Cop()
        for order in order_list:
            ord_ele = dconv.conv_TS_Order_2_cop_order(order=order)

            cop.data_map[ts_cop_pb2.Cop.FidNum.Order].order_data.order_list.append(ord_ele)

        if is_save_to_cache and self.__is_active_bg:
            # save to output cache
            self.__output_dcache_m.save_orders(cache_name=self.__sender_name, orders=order_list)

        self.__ts_datactrl_pub.send_cop(msg_id=msg_id
                                        , msg_type=ts_cop_pb2.Cop.MsgType.Strategy
                                        , topic=self.__pub_topic
                                        , cop=cop)

    def send_trade(self, msg_id: str, trade_list: List[TS_Trade], is_save_to_cache: bool = True) -> None:
        cop = ts_cop_pb2.Cop()
        for trd in trade_list:
            trd_ele = dconv.conv_TS_Trade_2_cop_trade(trd=trd)

            cop.data_map[ts_cop_pb2.Cop.FidNum.Trade].trade_data.trade_list.append(trd_ele)

        if is_save_to_cache and self.__is_active_bg:
            # save to output cache
            self.__output_dcache_m.save_trades(cache_name=self.__sender_name, trades=trade_list)

        self.__ts_datactrl_pub.send_cop(msg_id=msg_id
                                        , msg_type=ts_cop_pb2.Cop.MsgType.Strategy
                                        , topic=self.__pub_topic
                                        , cop=cop)

    def send_bg_cop(self, msg_id: str, msg_type: ts_cop_pb2.Cop.MsgType, cop: ts_cop_pb2.Cop) -> None:
        if not self.__is_active_bg:
            return
        self.__ts_datactrl_pub.send_cop(msg_id=msg_id
                                        , msg_type=msg_type
                                        , topic=self.__pub_topic
                                        , cop=cop)
    def send_int(self
                 , msg_id: str
                 , msg_type: ts_cop_pb2.Cop.MsgType
                 , fid_num: int
                 , fid_value: int
                 , time: datetime) -> None:
        cop = ts_cop_pb2.Cop()
        if 70001 <= fid_num and fid_num <= 80000:
            cop.data_map[fid_num].d_int32_data.value = fid_num
            cop.data_map[fid_num].d_int32_data.time = time.timestamp()
        elif 80001 <= fid_num and fid_num <= 90000:
            cop.data_map[fid_num].d_int64_data.value = fid_num
            cop.data_map[fid_num].d_int64_data.time = time.timestamp()
        else:
            return None

        self.__ts_datactrl_pub.send_cop(msg_id=msg_id
                                        , msg_type=msg_type
                                        , topic=self.__pub_topic
                                        , cop=cop)

    def send_double(self
                    , msg_id: str
                    , msg_type: ts_cop_pb2.Cop.MsgType
                    , fid_num: int
                    , fid_value: float
                    , time: datetime) -> None:
        cop = ts_cop_pb2.Cop()
        if 90001 <= fid_num and fid_num <= 100000:
            cop.data_map[fid_num].d_float_data.value = fid_num
            cop.data_map[fid_num].d_float_data.time = time.timestamp()
        elif 100001 <= fid_num and fid_num <= 110000:
            cop.data_map[fid_num].d_double_data.value = fid_num
            cop.data_map[fid_num].d_double_data.time = time.timestamp()
        else:
            return None

        self.__ts_datactrl_pub.send_cop(msg_id=msg_id
                                        , msg_type=msg_type
                                        , topic=self.__pub_topic
                                        , cop=cop)

    def send_string(self
                    , msg_id: str
                    , msg_type: ts_cop_pb2.Cop.MsgType
                    , fid_num: int
                    , fid_value: float
                    , time: datetime) -> None:
        cop = ts_cop_pb2.Cop()
        if 110001 <= fid_num and fid_num <= 120000:
            cop.data_map[fid_num].d_string_data.value = fid_num
            cop.data_map[fid_num].d_string_data.time = time.timestamp()
        else:
            return None

        self.__ts_datactrl_pub.send_cop(msg_id=msg_id
                                        , msg_type=msg_type
                                        , topic=self.__pub_topic
                                        , cop=cop)

    def send_bool(self
                  , msg_id: str
                  , msg_type: ts_cop_pb2.Cop.MsgType
                  , fid_num: int
                  , fid_value: float
                  , time: datetime) -> None:
        cop = ts_cop_pb2.Cop()
        if 120001 <= fid_num and fid_num <= 130000:
            cop.data_map[fid_num].d_bool_data.value = fid_num
            cop.data_map[fid_num].d_bool_data.time = time.timestamp()
        else:
            return None

        self.__ts_datactrl_pub.send_cop(msg_id=msg_id
                                        , msg_type=msg_type
                                        , topic=self.__pub_topic
                                        , cop=cop)

    def get_cur_sent_byte(self) -> int:
        return self.__ts_datactrl_pub.get_cur_sent_byte()

    def send_msg_list(self
                      , msg_id: str
                      , msg_type: ts_cop_pb2.Cop.MsgType
                      , fid_nums: List[int]
                      , fid_values: List[Any]
                      , time: datetime) -> None:
        # convert to cop format
        idx = 0
        for fid_num, fid_value in zip(fid_nums, fid_values):
            if fid_num in self.__data_convert_map:
                self.__data_convert_map[fid_num](fid_values=fid_values, idx=idx)

            idx += 1

        self.__ts_datactrl_pub.send_cop_list(msg_id=msg_id
                                             , msg_type=msg_type
                                             , fid_nums=fid_nums
                                             , fid_values=fid_values
                                             , topic=self.__pub_topic
                                             , timestamp=time.timestamp())

    def __convert_si(self, fid_values: List[Any], idx: int) -> None:
        fid_values[idx] = conv.conv_SI_2_cop_si(si=fid_values[idx])

    def __convert_order(self, fid_values: List[Any], idx: int):
        fid_values[idx] = conv.conv_TS_Order_2_cop_order(order=fid_values[idx])

    def __convert_trade(self, fid_values: List[Any], idx: int):
        fid_values[idx] = conv.conv_TS_Trade_2_cop_trade(trd=fid_values[idx])



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tweety.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tweety.proto',
  package='tweety',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0ctweety.proto\x12\x06tweety\"\\\n\x18RemoveStrategyConfig_Msg\x12\x15\n\rstrategy_name\x18\x01 \x01(\t\x12\x0f\n\x07is_live\x18\x02 \x01(\x08\x12\x18\n\x10strategy_id_list\x18\x03 \x03(\t\"#\n\x13RemoveOldConfig_Msg\x12\x0c\n\x04\x64\x61ys\x18\x01 \x01(\x05\"\x1c\n\x0b\x44ummy_Reply\x12\r\n\x05\x64ummy\x18\x01 \x01(\t\"?\n\x15GetStrategyConfig_Msg\x12\x15\n\rstrategy_name\x18\x01 \x01(\t\x12\x0f\n\x07is_live\x18\x02 \x01(\x08\";\n\x17GetStrategyConfig_Reply\x12\x0e\n\x06status\x18\x01 \x01(\t\x12\x10\n\x08\x63\x66g_list\x18\x02 \x03(\t\"2\n\x17GetStrategyConfigId_Msg\x12\x17\n\x0fstrategy_config\x18\x01 \x01(\t\";\n\x19GetStrategyConfigId_Reply\x12\x0e\n\x06status\x18\x01 \x01(\t\x12\x0e\n\x06\x63\x66g_id\x18\x02 \x01(\t\"1\n\x16SaveStrategyConfig_Msg\x12\x17\n\x0fstrategy_config\x18\x01 \x01(\t\"*\n\x18SaveStrategyConfig_Reply\x12\x0e\n\x06status\x18\x01 \x01(\t\"\x17\n\x04Ping\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x17\n\x04Pong\x12\x0f\n\x07message\x18\x01 \x01(\t2\xcf\x03\n\x06Tweety\x12Y\n\x13GetStrategyConfigId\x12\x1f.tweety.GetStrategyConfigId_Msg\x1a!.tweety.GetStrategyConfigId_Reply\x12V\n\x12SaveStrategyConfig\x12\x1e.tweety.SaveStrategyConfig_Msg\x1a .tweety.SaveStrategyConfig_Reply\x12S\n\x11GetStrategyConfig\x12\x1d.tweety.GetStrategyConfig_Msg\x1a\x1f.tweety.GetStrategyConfig_Reply\x12M\n\x14RemoveStrategyConfig\x12 .tweety.RemoveStrategyConfig_Msg\x1a\x13.tweety.Dummy_Reply\x12\x43\n\x0fRemoveOldConfig\x12\x1b.tweety.RemoveOldConfig_Msg\x1a\x13.tweety.Dummy_Reply\x12)\n\x0bHealthCheck\x12\x0c.tweety.Ping\x1a\x0c.tweety.Pongb\x06proto3'
)




_REMOVESTRATEGYCONFIG_MSG = _descriptor.Descriptor(
  name='RemoveStrategyConfig_Msg',
  full_name='tweety.RemoveStrategyConfig_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='tweety.RemoveStrategyConfig_Msg.strategy_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_live', full_name='tweety.RemoveStrategyConfig_Msg.is_live', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='strategy_id_list', full_name='tweety.RemoveStrategyConfig_Msg.strategy_id_list', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=116,
)


_REMOVEOLDCONFIG_MSG = _descriptor.Descriptor(
  name='RemoveOldConfig_Msg',
  full_name='tweety.RemoveOldConfig_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='days', full_name='tweety.RemoveOldConfig_Msg.days', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=118,
  serialized_end=153,
)


_DUMMY_REPLY = _descriptor.Descriptor(
  name='Dummy_Reply',
  full_name='tweety.Dummy_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dummy', full_name='tweety.Dummy_Reply.dummy', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=155,
  serialized_end=183,
)


_GETSTRATEGYCONFIG_MSG = _descriptor.Descriptor(
  name='GetStrategyConfig_Msg',
  full_name='tweety.GetStrategyConfig_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_name', full_name='tweety.GetStrategyConfig_Msg.strategy_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_live', full_name='tweety.GetStrategyConfig_Msg.is_live', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=185,
  serialized_end=248,
)


_GETSTRATEGYCONFIG_REPLY = _descriptor.Descriptor(
  name='GetStrategyConfig_Reply',
  full_name='tweety.GetStrategyConfig_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='tweety.GetStrategyConfig_Reply.status', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cfg_list', full_name='tweety.GetStrategyConfig_Reply.cfg_list', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=250,
  serialized_end=309,
)


_GETSTRATEGYCONFIGID_MSG = _descriptor.Descriptor(
  name='GetStrategyConfigId_Msg',
  full_name='tweety.GetStrategyConfigId_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_config', full_name='tweety.GetStrategyConfigId_Msg.strategy_config', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=311,
  serialized_end=361,
)


_GETSTRATEGYCONFIGID_REPLY = _descriptor.Descriptor(
  name='GetStrategyConfigId_Reply',
  full_name='tweety.GetStrategyConfigId_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='tweety.GetStrategyConfigId_Reply.status', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cfg_id', full_name='tweety.GetStrategyConfigId_Reply.cfg_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=363,
  serialized_end=422,
)


_SAVESTRATEGYCONFIG_MSG = _descriptor.Descriptor(
  name='SaveStrategyConfig_Msg',
  full_name='tweety.SaveStrategyConfig_Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='strategy_config', full_name='tweety.SaveStrategyConfig_Msg.strategy_config', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=424,
  serialized_end=473,
)


_SAVESTRATEGYCONFIG_REPLY = _descriptor.Descriptor(
  name='SaveStrategyConfig_Reply',
  full_name='tweety.SaveStrategyConfig_Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='tweety.SaveStrategyConfig_Reply.status', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=475,
  serialized_end=517,
)


_PING = _descriptor.Descriptor(
  name='Ping',
  full_name='tweety.Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='tweety.Ping.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=519,
  serialized_end=542,
)


_PONG = _descriptor.Descriptor(
  name='Pong',
  full_name='tweety.Pong',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='tweety.Pong.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=544,
  serialized_end=567,
)

DESCRIPTOR.message_types_by_name['RemoveStrategyConfig_Msg'] = _REMOVESTRATEGYCONFIG_MSG
DESCRIPTOR.message_types_by_name['RemoveOldConfig_Msg'] = _REMOVEOLDCONFIG_MSG
DESCRIPTOR.message_types_by_name['Dummy_Reply'] = _DUMMY_REPLY
DESCRIPTOR.message_types_by_name['GetStrategyConfig_Msg'] = _GETSTRATEGYCONFIG_MSG
DESCRIPTOR.message_types_by_name['GetStrategyConfig_Reply'] = _GETSTRATEGYCONFIG_REPLY
DESCRIPTOR.message_types_by_name['GetStrategyConfigId_Msg'] = _GETSTRATEGYCONFIGID_MSG
DESCRIPTOR.message_types_by_name['GetStrategyConfigId_Reply'] = _GETSTRATEGYCONFIGID_REPLY
DESCRIPTOR.message_types_by_name['SaveStrategyConfig_Msg'] = _SAVESTRATEGYCONFIG_MSG
DESCRIPTOR.message_types_by_name['SaveStrategyConfig_Reply'] = _SAVESTRATEGYCONFIG_REPLY
DESCRIPTOR.message_types_by_name['Ping'] = _PING
DESCRIPTOR.message_types_by_name['Pong'] = _PONG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RemoveStrategyConfig_Msg = _reflection.GeneratedProtocolMessageType('RemoveStrategyConfig_Msg', (_message.Message,), {
  'DESCRIPTOR' : _REMOVESTRATEGYCONFIG_MSG,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.RemoveStrategyConfig_Msg)
  })
_sym_db.RegisterMessage(RemoveStrategyConfig_Msg)

RemoveOldConfig_Msg = _reflection.GeneratedProtocolMessageType('RemoveOldConfig_Msg', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEOLDCONFIG_MSG,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.RemoveOldConfig_Msg)
  })
_sym_db.RegisterMessage(RemoveOldConfig_Msg)

Dummy_Reply = _reflection.GeneratedProtocolMessageType('Dummy_Reply', (_message.Message,), {
  'DESCRIPTOR' : _DUMMY_REPLY,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.Dummy_Reply)
  })
_sym_db.RegisterMessage(Dummy_Reply)

GetStrategyConfig_Msg = _reflection.GeneratedProtocolMessageType('GetStrategyConfig_Msg', (_message.Message,), {
  'DESCRIPTOR' : _GETSTRATEGYCONFIG_MSG,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.GetStrategyConfig_Msg)
  })
_sym_db.RegisterMessage(GetStrategyConfig_Msg)

GetStrategyConfig_Reply = _reflection.GeneratedProtocolMessageType('GetStrategyConfig_Reply', (_message.Message,), {
  'DESCRIPTOR' : _GETSTRATEGYCONFIG_REPLY,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.GetStrategyConfig_Reply)
  })
_sym_db.RegisterMessage(GetStrategyConfig_Reply)

GetStrategyConfigId_Msg = _reflection.GeneratedProtocolMessageType('GetStrategyConfigId_Msg', (_message.Message,), {
  'DESCRIPTOR' : _GETSTRATEGYCONFIGID_MSG,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.GetStrategyConfigId_Msg)
  })
_sym_db.RegisterMessage(GetStrategyConfigId_Msg)

GetStrategyConfigId_Reply = _reflection.GeneratedProtocolMessageType('GetStrategyConfigId_Reply', (_message.Message,), {
  'DESCRIPTOR' : _GETSTRATEGYCONFIGID_REPLY,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.GetStrategyConfigId_Reply)
  })
_sym_db.RegisterMessage(GetStrategyConfigId_Reply)

SaveStrategyConfig_Msg = _reflection.GeneratedProtocolMessageType('SaveStrategyConfig_Msg', (_message.Message,), {
  'DESCRIPTOR' : _SAVESTRATEGYCONFIG_MSG,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.SaveStrategyConfig_Msg)
  })
_sym_db.RegisterMessage(SaveStrategyConfig_Msg)

SaveStrategyConfig_Reply = _reflection.GeneratedProtocolMessageType('SaveStrategyConfig_Reply', (_message.Message,), {
  'DESCRIPTOR' : _SAVESTRATEGYCONFIG_REPLY,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.SaveStrategyConfig_Reply)
  })
_sym_db.RegisterMessage(SaveStrategyConfig_Reply)

Ping = _reflection.GeneratedProtocolMessageType('Ping', (_message.Message,), {
  'DESCRIPTOR' : _PING,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.Ping)
  })
_sym_db.RegisterMessage(Ping)

Pong = _reflection.GeneratedProtocolMessageType('Pong', (_message.Message,), {
  'DESCRIPTOR' : _PONG,
  '__module__' : 'tweety_pb2'
  # @@protoc_insertion_point(class_scope:tweety.Pong)
  })
_sym_db.RegisterMessage(Pong)



_TWEETY = _descriptor.ServiceDescriptor(
  name='Tweety',
  full_name='tweety.Tweety',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=570,
  serialized_end=1033,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetStrategyConfigId',
    full_name='tweety.Tweety.GetStrategyConfigId',
    index=0,
    containing_service=None,
    input_type=_GETSTRATEGYCONFIGID_MSG,
    output_type=_GETSTRATEGYCONFIGID_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SaveStrategyConfig',
    full_name='tweety.Tweety.SaveStrategyConfig',
    index=1,
    containing_service=None,
    input_type=_SAVESTRATEGYCONFIG_MSG,
    output_type=_SAVESTRATEGYCONFIG_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetStrategyConfig',
    full_name='tweety.Tweety.GetStrategyConfig',
    index=2,
    containing_service=None,
    input_type=_GETSTRATEGYCONFIG_MSG,
    output_type=_GETSTRATEGYCONFIG_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveStrategyConfig',
    full_name='tweety.Tweety.RemoveStrategyConfig',
    index=3,
    containing_service=None,
    input_type=_REMOVESTRATEGYCONFIG_MSG,
    output_type=_DUMMY_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveOldConfig',
    full_name='tweety.Tweety.RemoveOldConfig',
    index=4,
    containing_service=None,
    input_type=_REMOVEOLDCONFIG_MSG,
    output_type=_DUMMY_REPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='HealthCheck',
    full_name='tweety.Tweety.HealthCheck',
    index=5,
    containing_service=None,
    input_type=_PING,
    output_type=_PONG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_TWEETY)

DESCRIPTOR.services_by_name['Tweety'] = _TWEETY

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import sfdevtools.grpc_protos.eagle_pb2 as eagle__pb2


class EagleStub(object):
    """The eagle service definition.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.HealthCheck = channel.unary_unary(
                '/eagle.Eagle/HealthCheck',
                request_serializer=eagle__pb2.Ping.SerializeToString,
                response_deserializer=eagle__pb2.Pong.FromString,
                )
        self.GetDataByStrategyId = channel.unary_unary(
                '/eagle.Eagle/GetDataByStrategyId',
                request_serializer=eagle__pb2.GetDataByStrategyId_Msg.SerializeToString,
                response_deserializer=eagle__pb2.GetDataByStrategyId_Reply.FromString,
                )
        self.GetBacktestWarmupData = channel.unary_unary(
                '/eagle.Eagle/GetBacktestWarmupData',
                request_serializer=eagle__pb2.GetBacktestWarmupData_Msg.SerializeToString,
                response_deserializer=eagle__pb2.GetBacktestWarmupData_Reply.FromString,
                )
        self.GetUSLEIData = channel.unary_unary(
                '/eagle.Eagle/GetUSLEIData',
                request_serializer=eagle__pb2.GetUSLEIData_Msg.SerializeToString,
                response_deserializer=eagle__pb2.GetUSLEIData_Reply.FromString,
                )


class EagleServicer(object):
    """The eagle service definition.
    """

    def HealthCheck(self, request, context):
        """Unary - request and respond
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDataByStrategyId(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBacktestWarmupData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUSLEIData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_EagleServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'HealthCheck': grpc.unary_unary_rpc_method_handler(
                    servicer.HealthCheck,
                    request_deserializer=eagle__pb2.Ping.FromString,
                    response_serializer=eagle__pb2.Pong.SerializeToString,
            ),
            'GetDataByStrategyId': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDataByStrategyId,
                    request_deserializer=eagle__pb2.GetDataByStrategyId_Msg.FromString,
                    response_serializer=eagle__pb2.GetDataByStrategyId_Reply.SerializeToString,
            ),
            'GetBacktestWarmupData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBacktestWarmupData,
                    request_deserializer=eagle__pb2.GetBacktestWarmupData_Msg.FromString,
                    response_serializer=eagle__pb2.GetBacktestWarmupData_Reply.SerializeToString,
            ),
            'GetUSLEIData': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUSLEIData,
                    request_deserializer=eagle__pb2.GetUSLEIData_Msg.FromString,
                    response_serializer=eagle__pb2.GetUSLEIData_Reply.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'eagle.Eagle', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Eagle(object):
    """The eagle service definition.
    """

    @staticmethod
    def HealthCheck(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/eagle.Eagle/HealthCheck',
            eagle__pb2.Ping.SerializeToString,
            eagle__pb2.Pong.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDataByStrategyId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/eagle.Eagle/GetDataByStrategyId',
            eagle__pb2.GetDataByStrategyId_Msg.SerializeToString,
            eagle__pb2.GetDataByStrategyId_Reply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetBacktestWarmupData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/eagle.Eagle/GetBacktestWarmupData',
            eagle__pb2.GetBacktestWarmupData_Msg.SerializeToString,
            eagle__pb2.GetBacktestWarmupData_Reply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUSLEIData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/eagle.Eagle/GetUSLEIData',
            eagle__pb2.GetUSLEIData_Msg.SerializeToString,
            eagle__pb2.GetUSLEIData_Reply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import time
import threading

class HDTimer(object):
    def __init__(self, avg_size: int = 1):
        self.__avg_size = avg_size
        self.__lock = threading.Lock()
        self.__start_time = None
        self.__total_durationg = 0
        self.__window = list()

    def start_timer(self):
        with self.__lock:
            self.__start_time = time.time()

    def stop_timer(self):
        with self.__lock:
            diff_time = time.time() - self.__start_time
            self.__total_durationg += diff_time
            self.__window.append(diff_time)

            if len(self.__window) > self.__avg_size:
                self.__total_durationg -= self.__window[0]
                self.__window.pop(0)

    def get_avg_time(self):
        with self.__lock:
            return self.__total_durationg / max(min(len(self.__window), self.__avg_size), 1)

    def get_avg_size(self):
        return max(min(len(self.__window), self.__avg_size), 1)



import logging
import zmq
import threading
import signal
import math
from typing import Any, List, Dict, Union
import json

import sfdevtools.grpc_protos.ts_cop_pb2 as ts_cop_pb2
import sfdevtools.grpc_protos.ts_cop_pb2_grpc as ts_cop_pb2_grpc
import sfdevtools.grpc_protos.cop_tools as ct

class TSDataCtrl(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__pub_mutex: threading.Lock = threading.Lock()
        self.__sub_mutex: threading.Lock = threading.Lock()
        self.__publisher: zmq.sugar.socket.Socket = None
        self.__subscriber: zmq.sugar.socket.Socket = None
        self.__sender_name: str = ""
        self.__instance_id: int = 0
        self.__strategy_id: str = None
        self.__seq_num: int = 0
        self.__cb_fun = None
        self.__sub_main_thread: threading.Thread = None
        self.__is_subscription_run: bool = False
        self.__sub_timeout_ms: int = 100
        self.__max_byte_limit: int = 1024 * 1024 * 100 # 100 MB
        self.__cur_byte_sent: int = 0
        self.__is_send_limit_alert: bool = True
        self.__is_limit_sending: bool = False

        self.__process_fid_map_smart = {
            7: self.__process_d_int32
            , 8: self.__process_d_int64
            , 9: self.__process_d_float
            , 10: self.__process_d_double
            , 11: self.__process_d_string
            , 12: self.__process_d_bool
            , ts_cop_pb2.Cop.FidNum.CI: self.__process_ci_smart
            , ts_cop_pb2.Cop.FidNum.SI: self.__process_si_smart
            , ts_cop_pb2.Cop.FidNum.Order: self.__process_order_smart
            , ts_cop_pb2.Cop.FidNum.Trade: self.__process_trade_smart
            , ts_cop_pb2.Cop.FidNum.Order_Snap: self.__process_order_smart
        }
        self.__put_value_2_cop_map = {
            7: self.__put_value_d_int32
            , 8: self.__put_value_d_int64
            , 0: self.__put_value_d_float
            , 10: self.__put_value_d_double
            , 11: self.__put_value_d_string
            , 12: self.__put_value_d_bool
            , ts_cop_pb2.Cop.FidNum.CI: self.__put_value_ci
            , ts_cop_pb2.Cop.FidNum.SI: self.__put_value_si
            , ts_cop_pb2.Cop.FidNum.Order: self.__put_value_order
            , ts_cop_pb2.Cop.FidNum.Trade: self.__put_value_trade
            , ts_cop_pb2.Cop.FidNum.Order_Snap: self.__put_value_order
        }
        self.__process_fid_map = {
            1: self.__process_int32
            , 2: self.__process_int64
            , 3: self.__process_float
            , 4: self.__process_double
            , 5: self.__process_string
            , 7: self.__process_bool
            , ts_cop_pb2.Cop.FidNum.CI: self.__process_ci
            , ts_cop_pb2.Cop.FidNum.SI: self.__process_si
            , ts_cop_pb2.Cop.FidNum.Order: self.__process_order
            , ts_cop_pb2.Cop.FidNum.Trade: self.__process_trade
            , ts_cop_pb2.Cop.FidNum.Order_Snap: self.__process_order
        }

    def init_component(self
                       , logger: logging.Logger):
        self.__logger = logger

    def init_publish(self
                     , pub_host: str
                     , pub_port: str
                     , sender_name: str
                     , strategy_id: str
                     , instance_id: int):
        ctx = zmq.Context.instance()
        self.__publisher = ctx.socket(zmq.PUB)
        publisher_url = f"tcp://{pub_host}:{pub_port}"
        self.__logger.info(f"publisher_url: {publisher_url}")
        self.__publisher.connect(publisher_url)
        self.__logger.info("socket connect done")

        self.__sender_name = sender_name
        self.__instance_id = instance_id
        self.__strategy_id = strategy_id

    def init_subscribe(self
                       , sub_host: str
                       , sub_port: str
                       , timeout_ms: int = 100):
        ctx = zmq.Context.instance()
        self.__subscriber = ctx.socket(zmq.SUB)
        subscriber_url = f"tcp://{sub_host}:{sub_port}"
        self.__logger.info(f"subscriber_url: {subscriber_url}")
        self.__subscriber.connect(subscriber_url)
        self.__logger.info("socket connect done")

        self.__sub_timeout_ms = timeout_ms

    def set_max_send_limit(self, max_limit: int) -> None:
        with self.__pub_mutex:
            self.__max_byte_limit = max_limit

    def reset_sent_byte(self) -> None:
        with self.__pub_mutex:
            self.__cur_byte_sent = 0

    def get_cur_sent_byte(self) -> int:
        with self.__pub_mutex:
            return self.__cur_byte_sent

    def reg_sub_topic(self, topic: str):
        self.__logger.info(f"Subscribe topic: {topic}")
        self.__subscriber.setsockopt(zmq.SUBSCRIBE, str.encode(topic))

    def start_subscription(self, mode: str = "simple"):
        with self.__sub_mutex:
            self.__is_subscription_run = True
            if mode == "simple":
                self.__sub_main_thread = threading.Thread(target=self.sub_main_simple)
            elif mode == "smart":
                self.__sub_main_thread = threading.Thread(target=self.sub_main_smart)
            else:
                self.__sub_main_thread = threading.Thread(target=self.sub_main_smart)
            self.__sub_main_thread.start()

            signal.signal(signal.SIGINT, self.__cleanup)

    def stop_subscription(self):
        with self.__sub_mutex:
            self.__is_subscription_run = False

        self.__sub_main_thread.join()

    def reg_callback(self, cb_fun):
        with self.__sub_mutex:
            self.__cb_fun = cb_fun

    def unreg_callback(self):
        with self.__sub_mutex:
            self.__cb_fun = None

    def send_cop(self, msg_id: str, msg_type: ts_cop_pb2.Cop.MsgType, topic: str, cop: ts_cop_pb2.Cop):
        cop.msg_id = msg_id
        cop.msg_type = msg_type
        cop.sender = self.__sender_name
        cop.instance_id = self.__instance_id
        cop.strategy_id = self.__strategy_id
        with self.__pub_mutex:
            cop.seq_num = self.__seq_num
            if self.__is_limit_sending and not self.__is_can_send(cop_size=cop.ByteSize()
                                                                  , max_byte=self.__max_byte_limit
                                                                  , cur_byte=self.__cur_byte_sent
                                                                  , is_send_alert=self.__is_send_limit_alert):
                return
            self.__cur_byte_sent += cop.ByteSize()

            self.__seq_num += 1
            self.__publisher.send_multipart([str.encode(topic), cop.SerializeToString()])

    def send_cop_list(self
                      , msg_id: str
                      , msg_type: ts_cop_pb2.Cop.MsgType
                      , fid_nums: List[int]
                      , fid_values: List[Any]
                      , topic: str
                      , timestamp: float) -> None:
        cop = ts_cop_pb2.Cop()
        cop.msg_id = msg_id
        cop.msg_type = msg_type
        cop.sender = self.__sender_name
        cop.instance_id = self.__instance_id
        cop.strategy_id = self.__strategy_id

        # put data into cop
        for fid_num, fid_value in zip(fid_nums, fid_values):
            if 50001 <= fid_num and fid_num <= 60000:
                if fid_num not in self.__put_value_2_cop_map:
                    continue
                self.__put_value_2_cop_map[fid_num](cop=cop, fid_num=fid_num, fid_value=fid_value)
            else:
                num = ct.get_fid_group_num(fid_num=fid_num)
                if num not in self.__process_fid_map_smart:
                    continue
                self.__put_value_2_cop_map[num](cop=cop, fid_num=fid_num, fid_value=fid_value, timestamp=timestamp)

        with self.__pub_mutex:
            cop.seq_num = self.__seq_num
            if self.__is_limit_sending and not self.__is_can_send(cop_size=cop.ByteSize()
                                                                  , max_byte=self.__max_byte_limit
                                                                  , cur_byte=self.__cur_byte_sent
                                                                  , is_send_alert=self.__is_send_limit_alert):
                return
            self.__cur_byte_sent += cop.ByteSize()

            self.__seq_num += 1
            self.__publisher.send_multipart([str.encode(topic), cop.SerializeToString()])

    def reset_msg_seq_num(self):
        with self.__pub_mutex:
            self.__seq_num = 0

    def sub_main_smart(self):
        self.__logger.info("Start")

        poller = zmq.Poller()
        poller.register(self.__subscriber, zmq.POLLIN)
        while self.__is_subscription():
            sockets = dict(poller.poll(self.__sub_timeout_ms))

            if not self.__is_subscription():
                break

            # get and process data
            if self.__subscriber in sockets:
                topic_encoded, data = self.__subscriber.recv_multipart()

                with self.__sub_mutex:
                    if self.__cb_fun is not None:
                        cop = ts_cop_pb2.Cop()
                        cop.ParseFromString(data)
                        topic = topic_encoded.decode("utf-8")

                        # tsdata
                        for fid_num, fid_value in cop.data_map.items():
                            if 50001 <= fid_num and fid_num <= 60000:
                                if fid_num not in self.__process_fid_map_smart:
                                    continue
                                self.__process_fid_map_smart[fid_num](topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                            else:
                                num = ct.get_fid_group_num(fid_num=fid_num)
                                if num not in self.__process_fid_map_smart:
                                    continue
                                self.__process_fid_map_smart[num](topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

        self.__logger.info("End")

    def sub_main_extract(self):
        self.__logger.info("Start")

        poller = zmq.Poller()
        poller.register(self.__subscriber, zmq.POLLIN)
        while self.__is_subscription():
            sockets = dict(poller.poll(self.__sub_timeout_ms))

            if not self.__is_subscription():
                break

            # get and process data
            if self.__subscriber in sockets:
                topic_encoded, data = self.__subscriber.recv_multipart()

                with self.__sub_mutex:
                    if self.__cb_fun is not None:
                        cop = ts_cop_pb2.Cop()
                        cop.ParseFromString(data)
                        topic = topic_encoded.decode("utf-8")
                        # int32
                        for fid_num, fid_value in cop.int32_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                        # int64
                        for fid_num, fid_value in cop.int64_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                        # float
                        for fid_num, fid_value in cop.float_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                        # double
                        for fid_num, fid_value in cop.double_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                        # string
                        for fid_num, fid_value in cop.string_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                        # bool
                        for fid_num, fid_value in cop.bool_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                        # order
                        for fid_num, fid_value in cop.order_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                        # trade
                        for fid_num, fid_value in cop.trade_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                        # si
                        for fid_num, fid_value in cop.si_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)
                        # ci
                        for fid_num, fid_value in cop.ci_map.items():
                            self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

        self.__logger.info("End")

    def sub_main_simple(self):
        self.__logger.info("Start")

        poller = zmq.Poller()
        poller.register(self.__subscriber, zmq.POLLIN)
        while self.__is_subscription():
            sockets = dict(poller.poll(self.__sub_timeout_ms))

            if not self.__is_subscription():
                break

            # get and process data
            if self.__subscriber in sockets:
                topic, data = self.__subscriber.recv_multipart()

                with self.__sub_mutex:
                    if self.__cb_fun is not None:
                        cop = ts_cop_pb2.Cop()
                        cop.ParseFromString(data)
                        self.__cb_fun(topic.decode("utf-8"), cop)

            if not self.__is_subscription():
                break

        self.__logger.info("End")

    def __process_int32(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.int32_data)

    def __process_int64(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.int64_data)

    def __process_float(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.float_data)

    def __process_double(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.double_data)

    def __process_string(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.string_data)

    def __process_bool(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.bool_data)

    def __process_d_int32(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_d_int64(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_d_float(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_d_double(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_d_string(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_d_bool(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_ci(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_si(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_order(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_trade(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value)

    def __process_ci_smart(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.ci_data)

    def __process_si_smart(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.si_data)

    def __process_order_smart(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.order_data)

    def __process_trade_smart(self, topic: str, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any):
        self.__cb_fun(topic=topic, cop=cop, fid_num=fid_num, fid_value=fid_value.trade_data)

    def __cleanup(self, signum, frame):
        self.stop_subscription()
        exit(0)

    def __is_subscription(self):
        with self.__sub_mutex:
            return self.__is_subscription_run

    def __is_can_send(self, cop_size: int, max_byte: int, cur_byte: int, is_send_alert: bool) -> bool:
        if cur_byte + cop_size > max_byte:
            if is_send_alert:
                self.__logger.error(f"Cannot send cop due to size limit. sent: {cur_byte} max_byte: {max_byte} cop_size: {cop_size}")
                is_send_alert = False
                return False
            else:
                return False
        else:
            return True

    def __put_value_d_int32(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: int, timestamp: float):
        cop.data_map[fid_num].d_int32_data.time = timestamp
        cop.data_map[fid_num].d_int32_data.value = fid_value

    def __put_value_d_int64(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: int, timestamp: float):
        cop.data_map[fid_num].d_int64_data.time = timestamp
        cop.data_map[fid_num].d_int64_data.value = fid_value

    def __put_value_d_float(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: float, timestamp: float):
        cop.data_map[fid_num].d_float_data.time = timestamp
        cop.data_map[fid_num].d_float_data.value = fid_value

    def __put_value_d_double(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: float, timestamp: float):
        cop.data_map[fid_num].d_double_data.time = timestamp
        cop.data_map[fid_num].d_double_data.value = fid_value

    def __put_value_d_string(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: str, timestamp: float):
        cop.data_map[fid_num].d_string_data.time = timestamp
        cop.data_map[fid_num].d_string_data.value = fid_value

    def __put_value_d_bool(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: Any, timestamp: float):
        cop.data_map[fid_num].d_bool_data.time = timestamp
        cop.data_map[fid_num].d_bool_data.value = fid_value

    def __put_value_ci(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: List[Dict[Any, Any]]):
        for ci in fid_value: # ci_list:
            cop.data_map[fid_num].ci_data.ci_list.append(ts_cop_pb2.CI(value=json.dumps(ci)))

    def __put_value_si(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: List[ts_cop_pb2.SI]):
        for si in fid_value: # si_list:
            cop.data_map[fid_num].si_data.si_list.append(si)

    def __put_value_order(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: List[ts_cop_pb2.TSOrder]):
        for order in fid_value: # order_list:
            cop.data_map[fid_num].order_data.order_list.append(order)

    def __put_value_trade(self, cop: ts_cop_pb2.Cop, fid_num: int, fid_value: List[ts_cop_pb2.TSTrade]):
        for trd in fid_value: # trade_list:
            cop.data_map[fid_num].trade_data.trade_list.append(trd)



import logging
import boto3
from botocore.config import Config
from typing import List, Dict, Any, Union
from datetime import datetime, timedelta, date
import threading
import time
import pandas as pd

# https://github.com/awslabs/amazon-timestream-tools/blob/7d01250e8cda6a3da94150d1df6c125f70a017f3/sample_apps/python/CsvIngestionExample.py
class AWSTimeSeriesStorage(object):
    def __init__(self):
        self.__logger: logging.Logger = None
        self.__write_client: boto3.Session.client = None
        self.__query_client: boto3.Session.client = None
        self.__batch_size: int = 100
        self.__db_name: str = None
        self.__tb_name: str = None
        self.__mutex: threading.Lock = threading.Lock()

    def init_component(self
                       , logger: logging.Logger
                       , db_name: str
                       , tb_name: str
                       , batch_size: int = 100
                       , read_timeout: int = 20
                       , max_pool_connections: int = 5000
                       , max_attempts: int = 10) -> None:
        self.__logger = logger
        self.__batch_size = batch_size
        self.__db_name = db_name
        self.__tb_name = tb_name

        session = boto3.Session()
        self.__write_client = session.client("timestream-write", config=Config(read_timeout=read_timeout
                                                                               , max_pool_connections=max_pool_connections
                                                                               , retries={"max_attempts": max_attempts}))
        self.__query_client = session.client("timestream-query")

    def save_records(self, data_records: List[Any]):
        current_time = self._current_milli_time()
        with self.__mutex:
            records: List = list()
            # counter = 0
            for data in data_records:
                dimensions: List[Dict[str, str]] = data[0]
                measure_name: str = data[1]
                measure_value: str = data[2]
                measure_value_type: str = data[3]
                time_str: str = data[4]

                # record_time = current_time - (counter * 50)
                record = {
                    "Dimensions": dimensions
                    , "MeasureName": measure_name # "Any"
                    , "MeasureValue": measure_value # "0.01"
                    , "MeasureValueType": measure_value_type # "DOUBLE"
                    , "Time": time_str
                    # , "Time": str(record_time)
                    # , "Time": str(datetime.utcnow().timestamp())
                   }

                self.__logger.info(record)
                records.append(record)
                # counter = counter + 1

                if len(records) == self.__batch_size:
                    self._submit_batch(records=records, db_name=self.__db_name, tb_name=self.__tb_name)
                    records = list()

            if len(records) != 0:
                self._submit_batch(records=records, db_name=self.__db_name, tb_name=self.__tb_name)

    def get_records(self, query: str):
        q_data = self.__query_client.query(QueryString=query)

        header: List[str] = list()
        for col_ele in q_data["ColumnInfo"]:
            header.append(col_ele["Name"])
            # dtypes.append(col_ele["Type"]["ScalarType"]) # TODO, can map datatype from aws to pandas


        data_list: List[List] = list()
        for row_ele in q_data["Rows"]:
            data: List = list()
            for ele in row_ele["Data"]:
                for k, v in ele.items():
                    if k == "NullValue":
                        data.append(None)
                    else:
                        data.append(v)

            data_list.append(data)

        return pd.DataFrame(data_list, columns=header)

    def _submit_batch(self
                      , records: List
                      , db_name: str
                      , tb_name: str) -> None:
        result = self.__write_client.write_records(DatabaseName=db_name
                                                   , TableName=tb_name
                                                   , Records=records
                                                   , CommonAttributes={})

    @staticmethod
    def _current_milli_time():
        return int(round(time.time() * 1000))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: falcon.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='falcon.proto',
  package='falcon',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0c\x66\x61lcon.proto\x12\x06\x66\x61lcon\"\x17\n\x04Ping\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x17\n\x04Pong\x12\x0f\n\x07message\x18\x01 \x01(\t23\n\x06\x46\x61lcon\x12)\n\x0bHealthCheck\x12\x0c.falcon.Ping\x1a\x0c.falcon.Pongb\x06proto3'
)




_PING = _descriptor.Descriptor(
  name='Ping',
  full_name='falcon.Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='falcon.Ping.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=47,
)


_PONG = _descriptor.Descriptor(
  name='Pong',
  full_name='falcon.Pong',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='falcon.Pong.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=49,
  serialized_end=72,
)

DESCRIPTOR.message_types_by_name['Ping'] = _PING
DESCRIPTOR.message_types_by_name['Pong'] = _PONG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Ping = _reflection.GeneratedProtocolMessageType('Ping', (_message.Message,), {
  'DESCRIPTOR' : _PING,
  '__module__' : 'falcon_pb2'
  # @@protoc_insertion_point(class_scope:falcon.Ping)
  })
_sym_db.RegisterMessage(Ping)

Pong = _reflection.GeneratedProtocolMessageType('Pong', (_message.Message,), {
  'DESCRIPTOR' : _PONG,
  '__module__' : 'falcon_pb2'
  # @@protoc_insertion_point(class_scope:falcon.Pong)
  })
_sym_db.RegisterMessage(Pong)



_FALCON = _descriptor.ServiceDescriptor(
  name='Falcon',
  full_name='falcon.Falcon',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=74,
  serialized_end=125,
  methods=[
  _descriptor.MethodDescriptor(
    name='HealthCheck',
    full_name='falcon.Falcon.HealthCheck',
    index=0,
    containing_service=None,
    input_type=_PING,
    output_type=_PONG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_FALCON)

DESCRIPTOR.services_by_name['Falcon'] = _FALCON

# @@protoc_insertion_point(module_scope)
